@@ -3,6 +3,7 @@ Dummy
 """
 
 # pylint: disable=missing-function-docstring,import-outside-toplevel,invalid-name,unused-argument,protected-access,c-extension-no-member,consider-using-f-string,import-error,too-many-function-args
+# mypy: ignore-errors
 
 import os
 import re
@@ -71,6 +72,7 @@ def set_env_var_cmd(name: str, value: str) -> None:
         _print(f"Error happened while setting {name}={value}")
         _print(_try_decode(completed_proc.stdout))
     assert value in get_env_var(name), f"Failed to set {name}={value}"  # type: ignore
+    broadcast_changes()
 
 
 def unset_env_var_cmd(name: str) -> None:
@@ -119,23 +121,62 @@ def parse_paths(path_str: str) -> list[str]:
     return paths
 
 
+def set_env_path_registry(new_path: str, verbose=False):
+    if verbose:
+        print(f"&&& Setting {new_path} to Windows PATH")
+    try:
+        with winreg.OpenKey(
+            winreg.HKEY_LOCAL_MACHINE,
+            "SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment",
+            0,
+            winreg.KEY_ALL_ACCESS,
+        ) as key:
+            # Set the new value of the Path key
+            winreg.SetValueEx(key, "PATH", 0, winreg.REG_EXPAND_SZ, new_path)
+            broadcast_changes()
+    except Exception:  # pylint: disable=broad-except
+        print("Failed to add path to registry")
+
+
+def get_env_path_registry(verbose=False) -> str:
+    if verbose:
+        print("&&& Getting Windows PATH")
+
+    with winreg.OpenKey(
+        winreg.HKEY_LOCAL_MACHINE,
+        "SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment",
+        0,
+        winreg.KEY_READ,
+    ) as key:
+        # Get the value of the Path key
+        path = winreg.QueryValueEx(key, "PATH")[0]
+        return path
+
+
 def add_env_path(new_path: str, verbose=False):
     new_path = str(new_path)
     new_path = new_path.replace("/", "\\")
     if verbose:
-        print(f"&&& Adding {new_path} to Windows PATH")
-    prev_paths = parse_paths(get_reg_env_path())
-    if new_path in prev_paths:
+        print(f"&&& Adding {new_path} to Windows PATH:")
+    current_path = parse_paths(get_env_path_registry())
+    if verbose:
+        print(f"Current PATH: {current_path}")
+    if verbose and new_path in current_path:
         print(f"{new_path} already in PATH")
-        return
-    sep = os.path.pathsep
-    prev_path_str = sep.join(prev_paths)
-    new_path_str = f"{str(new_path)}{sep}{prev_path_str}"
-    set_env_var_cmd("PATH", new_path_str)
-    os.environ["PATH"] = new_path + sep + os.environ["PATH"]
+    else:
+        current_path.insert(0, new_path)
+        current_path_str = os.path.pathsep.join(current_path)
+        set_env_path_registry(current_path_str, verbose=False)
+    os_environ_paths = parse_paths(os.environ["PATH"])
+    if verbose and new_path in os_environ_paths:
+        print(f"{new_path} already in os.environ['PATH']")
+    else:
+        os_environ_paths.insert(0, new_path)
+        new_env_path_str = os.path.pathsep.join(os_environ_paths)
+        os.environ["PATH"] = new_env_path_str
 
 
-def set_env_var(var_name: str, var_value: str, verbose=True):
+def set_env_var(var_name: str, var_value: str, verbose=False):
     var_name = str(var_name)
     var_value = str(var_value)
     if verbose:
@@ -144,7 +185,7 @@ def set_env_var(var_name: str, var_value: str, verbose=True):
     os.environ[var_name] = var_value
 
 
-def unset_env_var(var_name: str, verbose=True):
+def unset_env_var(var_name: str, verbose=False):
     var_name = str(var_name)
     if verbose:
         print(f"$$$ Unsetting {var_name}")
@@ -157,17 +198,20 @@ def unset_env_var(var_name: str, verbose=True):
 def remove_env_path(path_to_remove: str, verbose=False):
     # convert / to \\ for Windows
     path_to_remove = path_to_remove.replace("/", "\\")
-    path_str = get_reg_env_path()
+    path_str = get_env_path_registry()
     if path_to_remove not in path_str:
         if verbose:
-            print(f"{path_to_remove} not in PATH")
+            print(f"{path_to_remove} not in PATH which is\n{path_str}")
         return
     paths = parse_paths(path_str)
     paths = [path for path in paths if path != path_to_remove]
     sep = os.path.pathsep
     new_path_str = sep.join(paths)
-    os.environ["PATH"] = new_path_str
-    set_env_var_cmd("PATH", new_path_str)
+    set_env_path_registry(new_path_str, verbose=verbose)
+    os_environ_paths = parse_paths(os.environ["PATH"])
+    os_environ_paths = [path for path in os_environ_paths if path != path_to_remove]
+    new_env_path_str = sep.join(os_environ_paths)
+    os.environ["PATH"] = new_env_path_str
 
 
 def main():
